#include	"netlib.h"
#include	<stdio.h>

#define MSL 512
int	Initialized = FALSE;

WSADATA	wdata;

//
// NetLib functions by Plastique:
//

//----------------------------------------------------------------------------------
void	WSA_Initialize()
{
		WSAStartup(0x101, &wdata);		// We don't need any bells and whistles
										// from winsock 2
		Initialized = TRUE;
}

//----------------------------------------------------------------------------------
SOCKET	TCP_CreateConnection(const struct in_addr con_addr, int port)
{
SOCKET	sock;
struct	sockaddr_in	addr;

		if (!Initialized) WSA_Initialize();

		sock = socket(AF_INET, SOCK_STREAM, 0);
		if (sock == INVALID_SOCKET) return 0;
		memset(&addr, 0, sizeof(struct sockaddr_in));

		addr.sin_family = AF_INET;
		addr.sin_addr.s_addr = con_addr.s_addr;
		addr.sin_port = htons(port);
		if (!connect(sock, (struct sockaddr *)&addr, sizeof(addr))) return sock;

		return 0;
}

//----------------------------------------------------------------------------------
// Create connection of string in format: "host:port"
SOCKET	TCP_CreateConnectionByName(char * addr, int default_port)
{
static	char	buf[MSL];
char	* port;
int		connect_port;
struct	hostent	*hp;
char	**p;
struct	in_addr in;
SOCKET	hndl;

		if (!Initialized) WSA_Initialize();

		strncpy(buf, addr, MSL);
		port = buf;
		while (*port && *port != ':') port++;
		if (!*port)
			connect_port = default_port;
		else
		{
			*port = '\0';
			connect_port = atoi(port+1);
		}
		if (isdigit(buf[0]))
		{
			in.s_addr = inet_addr(buf);
			return TCP_CreateConnection(in, connect_port);
		}
		else
			hp = gethostbyname(buf);

		if (!hp) return 0;


        for (p = hp->h_addr_list; *p!=0; p++)
		{
			memcpy(&in.s_addr, *p, sizeof(in.s_addr));
			hndl = TCP_CreateConnection(in, connect_port);
			if (hndl) return hndl;
		}
		return 0;
}
//----------------------------------------------------------------------------------
char	isreturn(char c)
{
		if (c == '\n' || c == '\r') return TRUE;
		return FALSE;
}
//----------------------------------------------------------------------------------
DWORD	WINAPI	NetLib_Thread(NetLib_Connection * conn)
{
fd_set	readfds;
struct	timeval	tv;
int		k;
char	buf[MSL];
char	* c, * lim, *start;

	if (conn->sock)
	{
		conn->Proc(conn, NETLIB_REASON_INITIALIZE, NULL, 0);
		conn->active = TRUE;
		conn->Proc(conn, NETLIB_REASON_CONNECT, NULL, 0);

		tv.tv_sec = 0;
		tv.tv_usec= 50000;		// 50 ms poll delay
        memset(buf,0,MSL);
		while (conn->active)
		{
			FD_ZERO(&readfds);
			FD_SET(conn->sock, &readfds);
			if (select((int)conn->sock + 1, &readfds, NULL, NULL, &tv) == -1)
			{
				conn->Proc(conn, NETLIB_REASON_DISCONNECT, NULL, 0);
				break;
			}

			if (FD_ISSET(conn->sock, &readfds))
			{
				k = recv(conn->sock, (char *)conn->buf + conn->buf_pos, RECV_BUFFER - conn->buf_pos, 0);

				if (k <= 0)
				{
					conn->Proc(conn, NETLIB_REASON_DISCONNECT, NULL, 0);
					break;
				}

				// Text mode
				if (conn->flags & TCP_FLAG_TEXT)
				{
					lim = (char *)(conn->buf + conn->buf_pos + k);
					c = (char *)conn->buf;

					conn->buf_pos = 0;

					while (c < lim)
					{
						start = c;

						while (c <= lim && !isreturn(*c)) c++;

						if (isreturn(*c))
						{
							while (c <= lim && isreturn(*c)) c++;
							strncpy(buf, start, (int)(c - start));
							conn->Proc(conn, NETLIB_REASON_DATA, buf, (int)(c - start));
						} else
						{
							memcpy(conn->buf, start, (int)(lim - start));
							conn->buf_pos = (int)(lim - start);
							break;
						}
					}
				} else
				{
				// Binary mode
						conn->Proc(conn, NETLIB_REASON_DATA, buf, k);
				}
			}
		}
	} else
		conn->Proc(conn, NETLIB_REASON_CONNECTION_FAILED, NULL, 0);
        if (!conn->active) conn->Proc(conn, NETLIB_REASON_DISCONNECT, NULL, 0);
		closesocket(conn->sock);
		free(conn);
		return 0;
}
//----------------------------------------------------------------------------------
int		NetLib_SendString(NetLib_Connection * conn, char * str)
{
		if (!conn) return 0;
		return send(conn->sock, str, (int)strlen(str), 0);
}
//----------------------------------------------------------------------------------
void	NetLib_SendXString(NetLib_Connection * conn, const char *fmt, ... )
{
va_list va_alist;
char buf[MSL];

	if (!conn) return;

	va_start (va_alist, fmt);
	_vsnprintf (buf, MSL, fmt, va_alist);
	va_end (va_alist);

	NetLib_SendString(conn, buf);
}
//----------------------------------------------------------------------------------
SOCKET	NetLib_CreateClient(char * host, NetLib_CallbackProc Callback, DWORD flags, int default_port)
{
NetLib_Connection	* conn;
SOCKET	sock;
DWORD	tid;
		
		sock = TCP_CreateConnectionByName(host, default_port);
//		if (!sock) return 0;

		conn = (NetLib_Connection *)malloc(sizeof(NetLib_Connection));
		if (!conn) return 0;

		conn->sock		= sock;
		conn->flags		= flags;
		conn->buf_pos	= 0;
		conn->Proc		= Callback;
		conn->active	= FALSE;

		CreateThread(NULL, 64000, (LPTHREAD_START_ROUTINE)NetLib_Thread, conn, 0, &tid);
		return sock;
}
//----------------------------------------------------------------------------------
